/*
 *     ApplicableChekcer.java  Aug 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import java.util.*;

public class ApplicableChekcer {

    public static boolean isApplicable(JavaCall ja, JavaMethod jm) {
        if (ja.getSignature().equals(jm.getSignature())) {
            return true;
        }

        if (ja.getNumArgument() == jm.getNumParameter() &&
            ja.getName().equals(jm.getName()) && isConvertible(ja, jm)) {
            return true;
        }

        return false;
    }        

    private static boolean isConvertible(JavaCall ja, JavaMethod jm) {
        for (int i = 0; i < jm.getNumParameter(); i++) {
            String argum = ja.getArgumentTypeFqn(i);
            String param = jm.getParameterTypeFqn(i);

            if (!isConvertible(argum, param)) {
                return false;
            }
        }
        return true;
    }

    public static boolean isApplicable(JavaMethod jm1, JavaMethod jm2) {
        if (jm1.getSignature().equals(jm2.getSignature())) {
            return true;
        }

        if (jm1.getNumParameter() == jm2.getNumParameter() &&
            jm1.getName().equals(jm2.getName()) && isConvertible(jm1, jm2)) {
            return true;
        }

        return false;
    }        

    public static boolean isConvertible(JavaMethod jm1, JavaMethod jm2) {
        for (int i = 0; i < jm1.getNumParameter(); i++) {
            String param1 = jm1.getParameterTypeFqn(i);
            String param2 = jm2.getParameterTypeFqn(i);

            if (!isConvertible(param1, param2)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isConvertible(String tp, String up) {
        if (tp == null || up == null) {
            return false;
        }

        if (tp.equals(up) || isSpeciallyConvertible(tp, up)) {
            return true;
        }

        while (tp.endsWith("[]") && up.endsWith("[]")) {
            tp = tp.substring(0, tp.lastIndexOf("["));
            up = up.substring(0, up.lastIndexOf("["));
        }
        if (tp.endsWith("[]") || up.endsWith("[]")) {
            return false;
        }

        if (TypeConversion.isPrimitiveType(tp) && TypeConversion.isPrimitiveType(up)) {
            return isConvertiblePrimitiveType(tp, up);
        }

        if (!TypeConversion.isPrimitiveType(tp) && !TypeConversion.isPrimitiveType(up)) {
            JavaClass tc = Repository.entryClass(tp);
            JavaClass uc = Repository.entryClass(up);
            return isConvertibleClassType(tc, uc);
        }
        return false;
    }

    private static boolean isSpeciallyConvertible(String tp, String up) {
        if (tp.equals("null")) {
            return true;
        }

        if (tp.endsWith("[]")) {
            if (up.equals("java.lang.Object") ||
                up.equals("java.lang.Cloneable") ||
                up.equals("java.lang.Serializable")) {
                return true;
            }
        }

        if (!TypeConversion.isPrimitiveType(tp) && up.equals("java.lang.Object")) {
            return true;
        }

        return false;
    }

    private static boolean isConvertiblePrimitiveType(String tp, String up) {
        String type = TypeConversion.wideningPrimitiveType(tp, up);
        if (type.equals(up)) {
            return true;
        }
        return false;
    }

    private static boolean isConvertibleClassType(JavaClass tc, JavaClass uc) {
        if (tc != null && uc != null) {
            if (tc.equals(uc)) {
                return true;
            }

            if (uc.getFqn().equals("java.lang.Object")) {
                return true;
            }

            JavaClassList ancestors = AncestorResolver.getAncestors(tc);
            if (ancestors.contains(uc)) {
                return true;
            }
        }
        return false;
    }
}
