/*
 *     GetFqn.java  Mar 26, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import java.io.*;

public class GetFqn {
    private static ArrayList candidates = new ArrayList();  // String
    private static HashMap fileMap = new HashMap();  // String/String
    private static HashMap fqnCache = new HashMap();  // String/String
    private static HashMap namesCache = new HashMap();  // String/ArrayList
    private static boolean canUseAncestors = false;

    private static HashMap classNameMap = new HashMap();  // String/ArrayList

    public static void canUseAncestors(boolean bool) {
        canUseAncestors = bool;
    }
    
    public static void init(JavaFile jfile) {
        candidates.clear();
        fileMap.clear();
        fqnCache.clear();
        namesCache.clear();
        classNameMap.clear();

        String fileName = jfile.getName();
        String packageName = jfile.getPackageName();
        ArrayList importNames = new ArrayList();

        Iterator it = jfile.getImportNames().iterator();
        while (it.hasNext()) {
            String importName = (String)it.next();
            if (importName.indexOf("*") == -1) {
                importNames.add(0, importName);
            } else {
                importNames.add(importName);
            }
        }

        // candidates.addAll(getFilesUnderProject(Repository.getJavaTexts()));
        if (packageName.length() == 0) {  // anonymous package
            candidates.addAll(getFilesInDir(jfile.getDirName()));
        } else {
            candidates.addAll(ClassPathFile.collectCandidates(packageName));
        }
        candidates.addAll(ClassPathFile.collectCandidates(importNames));

        collectAccessibleFiles(candidates, fileName, packageName);
    }

    private static void printCandidates(ArrayList files) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            System.out.println("CANDIDATE = " + name);
        }
    }

    /*
    private static ArrayList getFilesUnderProject(ArrayList texts) {
        ArrayList files = new ArrayList();

        Iterator it = texts.iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            files.add(text.getName());
        }
        return files;
    }
    */

    private static ArrayList getFilesInDir(String dirName) {
        ArrayList files = new ArrayList();

        File dir = new File(dirName);
        if (dir.isDirectory()) {
            String[] names = dir.list();
            for (int i = 0; i < names.length; i++) {
                String name = dirName + File.separator + names[i];

                File file = new File(name);
                if (file.isFile() &&
                    (name.endsWith(".java") || name.endsWith(".class"))) {
                    files.add(file.getPath());
                }
            }
        }
        return files;
    }

    private static void collectAccessibleFiles(ArrayList files,
                                               String fileName, String packegeName) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            addAccessibleFile(name, fileName, packegeName);
        }
    }

    private static void addAccessibleFile(String name, String fileName, String packageName) {
        if (name.endsWith(".java")) {
            ArrayList files = ClassPathFile.getJavaClassFiles(name);
            if (files != null) {
                Iterator it = files.iterator();

                while (it.hasNext()) {
                    ClassFileName cname = (ClassFileName)it.next();
                    if (fileName.equals(name) || cname.isPublic() ||
                        packageName.equals(cname.getPackageName())) {
                        String className = cname.getName();

                        if (getFileName(className) == null) {
                            fileMap.put(className, name);
                            putClassName(className);
                        }
                    }
                }
            }

        } else if (name.endsWith(".class")) {
            ClassFileName cname = ClassPathFile.getClassFile(name);
            if (cname != null && (cname.isPublic() ||
                 (packageName != null && packageName.equals(cname.getPackageName())))) {
                String className = cname.getName();

                if (getFileName(className) == null) {
                    fileMap.put(className, name);
                    putClassName(className);
                }
            }
        }
    }

    public static String getClassFileName(String fqn) {
        String name = getFileName(fqn);
        if (name != null) {
            return name;
        }
        return null;
    }        
    
    private static String getFileName(String fqn) {
        if (fqn != null) {
            int sep = fqn.indexOf("[");
            if (sep != -1) {
                fqn = fqn.substring(0, sep);
            }
            return (String)fileMap.get(fqn);
        }
        return null;
    }

    public static String getFqn(JavaClass jclass, String name) {
        if (name == null) {
            return null;
        }

        String qname = (String)fqnCache.get(name);
        if (qname != null) {
            return qname;
        }

        if (name.length() == 0) {
            return null;
        }

        if (name.equals("null") || TypeConversion.isPrimitiveType(name)) {
            fqnCache.put(name, name);
            return name;
        }

        if (name.indexOf(".") != -1) {
            String pathName = ClassPathFile.getClassFileName(name);
            if (pathName != null) {
                fqnCache.put(name, name);
                return name;
            }
        }

        qname = getMatchedName(name, jclass);
        if (qname != null) {
            fqnCache.put(name, qname);
            return qname;
        }

        while (name.indexOf(".") != -1) {
            int sep = name.lastIndexOf(".");
            name = name.substring(0, sep) + "$" + name.substring(sep + 1);
            
            qname = getMatchedName(name, jclass);
            if (qname != null) {
                return qname;
            }
        }

        return null;
    }

    private static String getMatchedName(String className, JavaClass jclass) {
        ArrayList names = getCandidateClassNames(jclass);
        String className1 = "." + className;
        String className2 = "$" + className;

        ArrayList fileList = getClassName(className);        
        if (fileList != null) {
            Iterator it = fileList.iterator();
            while (it.hasNext()) {
                String fqn = (String)it.next();
                String cname = "." + fqn;

                Iterator itn = names.iterator();
                while (itn.hasNext()) {
                    String name = (String)itn.next();

                    if (fqn.startsWith(name)) {
                        if (cname.endsWith(className1) || cname.endsWith(className2)) {
                            return fqn;
                        }
                    }
                }
            }

            it = fileList.iterator();
            while (it.hasNext()) {
                String fqn = (String)it.next();
                String cname = "." + fqn;
            
                if (cname.endsWith(className1)) {
                    return fqn;
                }
            }
        }

        return null;
    }

    private static ArrayList getCandidateClassNames(JavaClass jclass) {
        ArrayList names = (ArrayList)namesCache.get(jclass.getFqn());
        if (names != null) {
            return names;
        }
        names = new ArrayList();
        namesCache.put(jclass.getFqn(), names);
        
        JavaClassList classes = new JavaClassList();
        classes.add(jclass);
        classes.addAll(jclass.getAllOuterClasses());
        String packageName = jclass.getJavaFile().getPackageName();

        Iterator it1 = classes.iterator();
        while (it1.hasNext()) {
            JavaClass jc1 = (JavaClass)it1.next();
            names.add(jc1.getFqn());

            if (canUseAncestors) {
                Iterator it2 = AncestorResolver.getAncestors(jc1).iterator();
                while (it2.hasNext()) {
                    JavaClass jc2 = (JavaClass)it2.next();
                    
                    if (jc2.getJavaFile().getPackageName().equals(packageName)) { 
                        names.add(jc2.getFqn());
                    }
                }
            }
        }

        return names;
    }        

    private static void putClassName(String className) {
        String key = getKey(className);

        ArrayList classNameList = (ArrayList)classNameMap.get(key);
        if (classNameList == null) {
            classNameList = new ArrayList();
            classNameMap.put(key, classNameList);
        }
        classNameList.add(className);
    }

    private static ArrayList getClassName(String className) {
        String key = getKey(className);
        return (ArrayList)classNameMap.get(key);
    }

    private static String getKey(String name) {
        return name.substring(name.length());
    }
}
