/*
 *     Millx.java  May 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml.millx;
import org.jtool.platform.j2xml.*;
import java.io.*;

public class Millx {
    private static boolean isWindows = false;
    private static String HTMLDir = "html";
    private static final String JX3_DTD = "JX-model3.dtd";
    protected static String sapidDir = null;
    protected static String sdbDir;
    protected static final String is_valid_1 = ".isValidSDB-1";
    protected static final String is_valid_2 = ".isValidSDB-2";

    public static void init(String sdbd) {
        init(sdbd, null);
    }

    public static void init(String sdbd, String htmlDir) {
        sapidDir = SystemProperties.getSapidDir();
        sdbDir = sdbd;

        String options = "-sdbj -i";
        options = options + " -sdbd " + sdbDir;
        options = options + " -source 1.4";
        if (htmlDir != null) {
            options = options + " -sdbb -sdbH " + htmlDir;
        }
        if (System.getProperty("os.name").startsWith("Windows")) {
            isWindows = true;
            MillxWindows.init(options);

        } else {
            isWindows = false;
            MillxUnix.init(options);
        }
    }

    public static File run(File file) throws ParseException {
        String pathName = file.getPath();
        String xmlDir = pathName.substring(0, pathName.lastIndexOf(File.separator));
        return run(file, xmlDir);
    }

    public static File run(File file, String xmlDir) throws ParseException {
        if (file != null && file.length() != 0) {

            try {
                File valid1 = new File(sdbDir + File.separator + is_valid_1);
                File valid2 = new File(sdbDir + File.separator + is_valid_2);

                if (isWindows) {
                    MillxWindows.run(file.getPath(), xmlDir);
                } else {
                    MillxUnix.run(file.getPath(), xmlDir);
                }

                if (valid1.exists() && valid2.exists()) {
                    File dtdFile = new File(xmlDir + File.separator + JX3_DTD);
                    dtdFile.delete();
                    
                    File xmlFile = new File(file.getPath() + ".xml");
                    return xmlFile;
                }
            } catch(Exception e) { }
        }

        throw new ParseException("Fail to run millx: " + file.getName());
    }

    protected static String getMessage(InputStream is) throws IOException {
        StringBuffer buf = new StringBuffer();
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));

        String line;
        while ((line = reader.readLine()) != null) {
            buf.append(line + "\n");
        }
        reader.close();
        return new String(buf);
    }

    public static void main(String[] args) {
        if (args.length == 3 && args[0].equals("-sapiddir")) {

            SystemProperties.setApplicationDir(".");
            SystemProperties.setSapidDir(args[1]);
            String fileName = SystemProperties.getCanonicalPath(args[2]);

            try {
                Millx.init("SDB");
                Millx.run(new File(fileName));
                System.exit(0);

            } catch (ParseException e) { }
        }
        System.err.println("Usage: java Millx -spaiddir sapid_dir filename");
    }
}
